<?php

use ZondacryptoPay\Payment\ZondacryptoPay;
use ZondacryptoPay\Payment\Helper;
use ZondacryptoPay\Payment\Payment;
use ZondacryptoPay\Payment\Util;

/**
 * Class ControllerExtensionPaymentZondacryptopay
 */
class ControllerExtensionPaymentZondacryptopay extends Controller
{

	/**
	 * plugin version
	 */
	const VERSION = '1.2.0';

	/**
	 * @return mixed
	 */
	public function index()
	{

		$this->language->load('extension/payment/zondacryptopay');

		include_once(DIR_SYSTEM . 'library/payment-core/vendor/autoload.php');

		$this->load->model('setting/setting');

		$data = [
			'url_pay' => $this->url->link('extension/payment/zondacryptopay/pay', '', true),
		];

		return $this->load->view('extension/payment/zondacryptopay', $data);
	}

	/**
	 * initiate form payment, make payment
	 *
	 * @throws Exception
	 */
	public function pay()
	{
		include_once(DIR_SYSTEM . 'library/payment-core/vendor/autoload.php');

		$this->load->model('checkout/order');
		$this->language->load('extension/payment/zondacryptopay');

		$order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);

		$payment = new Payment(
			$this->config->get('payment_zondacryptopay_private_key'),
			$this->config->get('payment_zondacryptopay_public_key')
		);

		$base_url = $this->config->get('config_url');

		if(isset($this->request->server['HTTPS']) && (($this->request->server['HTTPS'] == 'on') || ($this->request->server['HTTPS'] == '1'))) {
			$base_url = $this->config->get('config_ssl');
		}

		try {

			$response = $payment->buildOrder(
				$order_info['currency_code'],
				$order_info['order_id'],
				round($order_info['total'], 2),
                $order_info['email'],
                $base_url . 'index.php?route=extension/payment/zondacryptopay/notification',
				$this->url->link('checkout/success'),
				$this->url->link('checkout/success')
			);
		} catch(Exception $e) {

			$lang = $this->getLanguage();

			$this->log->write('ZondacryptoPay: ' . Helper::getErrorMessage($e->getMessage(), $lang));

			$this->response->setOutput($this->load->view('extension/payment/zondacryptopay_redirect_fail', [
				'form' => Helper::getErrorMessage(Helper::ERROR_MESSAGE_FOR_CUSTOMER, $lang),
			]));

			return;
		}

		$this->model_checkout_order->addOrderHistory($order_info['order_id'], 01);

		$this->load->language('checkout/cart');
		$this->cart->clear();

		unset($this->session->data['vouchers']);
		unset($this->session->data['shipping_method']);
		unset($this->session->data['shipping_methods']);
		unset($this->session->data['payment_method']);
		unset($this->session->data['payment_methods']);
		unset($this->session->data['reward']);

		$this->response->setOutput($this->load->view('extension/payment/zondacryptopay_redirect',
			[
				'redirect' => $response->getUrl(),
                'button' => $this->language->get('text_submit')
			]
		));
	}

	/**
	 * Support incoming notification (and validate)
	 *
	 * @throws Exception
	 */
	public function notification()
	{

		include_once(DIR_SYSTEM . 'library/payment-core/vendor/autoload.php');

		if(!($payload_decoded = ZondacryptoPay::validateNotification())) {
			ZondacryptoPay::responseNotificationHeader(Util::HTTP_CODE_404);

			return;
		}

		if(!($transaction_api = ZondacryptoPay::checkTransaction($payload_decoded['paymentId'], $this->config->get('payment_zondacryptopay_private_key'), $this->config->get('payment_zondacryptopay_public_key')))) {
			ZondacryptoPay::responseNotificationHeader(Util::HTTP_CODE_404);

			return;
		}

		$this->load->model('checkout/order');

		if(!($order = $this->model_checkout_order->getOrder($payload_decoded['orderId']))) {
			ZondacryptoPay::responseNotificationHeader(Util::HTTP_CODE_404);

			return;
		}

		if(!($order['order_status_id'] === '1' || $order['order_status_id'] === '10')) {
			ZondacryptoPay::responseNotificationHeader(Util::HTTP_CODE_404);

			return;
		}

        if($transaction_api['data']['amountInDestinationCurrency'] !== (float) $order['total']) {
            ZondacryptoPay::responseNotificationHeader(Util::HTTP_CODE_404);

            return;
        }

        switch($transaction_api['data']['status']) {
            case Util::TRANSACTION_STATUS_COMPLETED:
                $this->model_checkout_order->addOrderHistory($order['order_id'], 02);
                break;
            case Util::TRANSACTION_STATUS_ACTION_REQUIRED:
                $this->language->load('extension/payment/zondacryptopay');
                $message = sprintf($this->language->get('text_action_required'),  $payload_decoded['travelRuleFormUrl'] ?? '');
                $this->model_checkout_order->addOrderHistory($order['order_id'], $order['order_status_id'], $message);
                break;
            default:
                ZondacryptoPay::responseNotificationHeader(Util::HTTP_CODE_404);

                return;
        }

		ZondacryptoPay::responseNotificationHeader(Util::HTTP_CODE_200);
	}

	/**
	 * @return string
	 */
	private function getLanguage()
	{
		$locale_code = strtolower($this->language->get('code'));

		if($locale_code === 'pl' || $locale_code === 'pl-pl') {
			return 'pl';
		}

		return 'en';
	}
}
